@@ -1,11 +1,12 @@
-from datetime import datetime, timedelta
-import time
 import re
+import time
 import pytz
 from copy import copy
+from datetime import datetime, timedelta
 
-from .timestring_re import TIMESTRING_RE
-from .string_to_number import string_to_number
+from timestring import TimestringInvalid
+from timestring.timestring_re import TIMESTRING_RE
+from timestring.string_to_number import string_to_number
 
 try:
     unicode
@@ -13,8 +14,14 @@ except NameError:
     unicode = str
     long = int
 
+CLEAN_NUMBER = re.compile(r"[\D]")
+
 class Date(object):
     def __init__(self, date, offset=None, start_of_week=None, tz=None, verbose=False):
+        if isinstance(date, Date):
+            self.date = copy(date.date)
+            return
+
         # The original request
         self._original = date
         if tz:
@@ -22,6 +29,13 @@ class Date(object):
 
         if date == 'infinity':
             self.date = 'infinity'
+
+        elif date == 'now':
+            self.date = datetime.now()
+
+        elif type(date) in (str, unicode) and re.match(r"\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d+-\d{2}", date):
+            self.date = datetime.strptime(date[:-3], "%Y-%m-%d %H:%M:%S.%f") - timedelta(hours=int(date[-3:]))
+
         else:
             # Determinal starting date.
             if type(date) in (str, unicode):
@@ -34,7 +48,7 @@ class Date(object):
                     if verbose:
                         print("Matches:\n", ''.join(["\t%s: %s\n" % (k, v) for k, v in date.items() if v]))
                 else:
-                    raise ValueError('Invlid date string >> %s' % date)
+                    raise TimestringInvalid('Invlid date string >> %s' % date)
 
                 date = dict((k, v if type(v) is str else v) for k, v in date.items() if v)
                 #print(_date, dict(map(lambda a: (a, date.get(a)), filter(lambda a: date.get(a), date))))
@@ -42,7 +56,7 @@ class Date(object):
             if isinstance(date, dict):
                 # Initial date.
                 new_date = datetime(*time.localtime()[:3])
-                if tz:
+                if tz and tz.zone != "UTC":
                     #
                     # The purpose here is to adjust what day it is based on the timezeone
                     #
@@ -126,16 +140,14 @@ class Date(object):
                         new_date = new_date - timedelta(days=1)
                     elif dow == 'tomorrow':
                         new_date = new_date + timedelta(days=1)
-                    elif dow == 'now':
-                        new_date = datetime(*time.localtime()[:6])
 
                 # !year
-                year = [date.get(key) for key in ('year', 'year_2', 'year_3', 'year_4', 'year_5', 'year_6') if date.get(key)]
+                year = [int(CLEAN_NUMBER.sub('', date[key])) for key in ('year', 'year_2', 'year_3', 'year_4', 'year_5', 'year_6') if date.get(key)]
                 if year:
-                    y = int(max(year))
-                    if len(str(y)) != 4:
-                        y += 2000 if y <= 40 else 1900
-                    new_date = new_date.replace(year=y)
+                    year = max(year)
+                    if len(str(year)) != 4:
+                        year += 2000 if year <= 40 else 1900
+                    new_date = new_date.replace(year=year)
 
                 # !month
                 month = [date.get(key) for key in ('month', 'month_1', 'month_2', 'month_3', 'month_4') if date.get(key)]
@@ -164,7 +176,9 @@ class Date(object):
                     new_date = new_date.replace(hour=int(max(hour)))
                     am = [date.get(key) for key in ('am', 'am_1') if date.get(key)]
                     if am and max(am) in ('p', 'pm'):
-                        new_date = new_date.replace(hour=int(max(hour))+12)
+                        h = int(max(hour))
+                        if h < 12:
+                            new_date = new_date.replace(hour=h+12)
                     # No offset because the hour was set.
                     offset = False
 
@@ -267,6 +281,14 @@ class Date(object):
         if self.date != 'infinity':
             return self.date.tzinfo
 
+    @tz.setter
+    def tz(self, tz):
+        if self.date != 'infinity':
+            if tz is None:
+                self.date = self.date.replace(tzinfo=None)
+            else:
+                self.date = self.date.replace(tzinfo=pytz.timezone(tz))
+
     def replace(self, **k):
         """Note returns a new Date obj"""
         if self.date != 'infinity':
@@ -318,22 +340,24 @@ class Date(object):
             new.date = new.date + timedelta(seconds=int(to))
             return new
 
-        raise ValueError('Invalid addition request')
+        raise TimestringInvalid('Invalid addition request')
 
     def __nonzero__(self):
         return True
 
     def __add__(self, to):
-        if self.date != 'infinity':
-            return copy(self).adjust(to)
+        if self.date == 'infinity':
+            return copy(self)
+        return copy(self).adjust(to)
 
     def __sub__(self, to):
-        if self.date != 'infinity':
-            if type(to) in (str, unicode):
-                to = to[1:] if to.startswith('-') else ('-'+to)
-            elif type(to) in (int, float, long):
-                to = to * -1
-            return copy(self).adjust(to)
+        if self.date == 'infinity':
+            return copy(self)
+        if type(to) in (str, unicode):
+            to = to[1:] if to.startswith('-') else ('-'+to)
+        elif type(to) in (int, float, long):
+            to = to * -1
+        return copy(self).adjust(to)
 
     def __format__(self, _):
         if self.date != 'infinity':
@@ -378,32 +402,27 @@ class Date(object):
 
     def __lt__(self, other):
         if self.date == 'infinity':
-            if isinstance(other, Date):
-                return other.date != 'infinity'
-            else:
-                from .Range import Range
-                if isinstance(other, Range):
-                    return other.end != 'infinity'
-                return other != 'infinity'
+            # infinity can never by less then a date
+            return False
+
+        if isinstance(other, Date):
+            if other.date == 'infinity':
+                return True
+            elif other.tz and self.tz is None:
+                return self.date.replace(tzinfo=other.tz) < other.date
+            elif self.tz and other.tz is None:
+                return self.date < other.date.replace(tzinfo=self.tz)
+            return self.date < other.date
         else:
-            if isinstance(other, Date):
-                if other.date == 'infinity':
-                    return True
-                elif other.tz and self.tz is None:
-                    return self.date.replace(tzinfo=other.tz) < other.date
-                elif self.tz and other.tz is None:
-                    return self.date < other.date.replace(tzinfo=self.tz)
-                return self.date < other.date
+            from .Range import Range
+            if isinstance(other, Range):
+                if other.end.tz and self.tz is None:
+                    return self.date.replace(tzinfo=other.end.tz) < other.end.date
+                elif self.tz and other.end.tz is None:
+                    return self.date < other.end.date.replace(tzinfo=self.tz)
+                return self.date < other.end.date
             else:
-                from .Range import Range
-                if isinstance(other, Range):
-                    if other.end.tz and self.tz is None:
-                        return self.date.replace(tzinfo=other.end.tz) < other.end.date
-                    elif self.tz and other.end.tz is None:
-                        return self.date < other.end.date.replace(tzinfo=self.tz)
-                    return self.date < other.end.date
-                else:
-                    return self.__lt__(Date(other, tz=self.tz))
+                return self.__lt__(Date(other, tz=self.tz))
 
     def __ge__(self, other):
         return self > other or self == other
@@ -412,19 +431,25 @@ class Date(object):
         return self < other or self == other
 
     def __eq__(self, other):
+        if isinstance(other, datetime):
+            other = Date(other)
         if isinstance(other, Date):
-           if other.tz and self.tz is None:
-               return self.date.replace(tzinfo=other.tz) == other.date
-           elif self.tz and other.tz is None:
-               return self.date == other.date.replace(tzinfo=self.tz)
-           return self.date == other.date
+            if other.date == 'infinity':
+                return self.date == 'infinity'
+
+            elif other.tz and self.tz is None:
+                return self.date.replace(tzinfo=other.tz) == other.date
+
+            elif self.tz and other.tz is None:
+                return self.date == other.date.replace(tzinfo=self.tz)
+
+            return self.date == other.date
         else:
             from .Range import Range
             if isinstance(other, Range):
                 return False
             else:
                 return self.__eq__(Date(other, tz=self.tz))
-        return False
 
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -432,14 +457,11 @@ class Date(object):
     def format(self, format_string='%x %X'):
         if self.date != 'infinity':
             return self.date.strftime(format_string)
+        else:
+            return 'infinity'
 
     def to_unixtime(self):
         if self.date != 'infinity':
             return time.mktime(self.date.timetuple())
-
-    def to_mysql(self):
-        if self.date != 'infinity':
-            return "date '%s'" % str(self.format("%Y-%m-%d %H:%M:%S"))
-
-    def to_postgresql(self):
-        return "'%s'::date" % str(self.date)
+        else:
+            return -1
